__version__ = "0.1.3"
__git_commiter_name__ = "Guy Rozendorn"
__git_commiter_email__ = "guy@rzn.co.il"
__git_branch__ = '(detached from v0.1.3)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = '12766fca54d65676f8a5d9a801d51cfd45e9062e'
__git_head_subject__ = 'HOSTDEV-944 fixing version resolution when switching between projects'
__git_head_message__ = '================================================================================\n\nHost Infrastructure / HOSTDEV-944\njish project <p> <c> fails with no such version if swtiched from other project\n\nType:              Bug                               Status:        Open                       Assignee: Guy Rozendorn\nPriority:          Minor                             Resolution:    Unresolved                 Reporter: Guy Rozendorn\nAffects Version/s:                                   Fix Version/s: 0.30\nComponents: jissue                                                                             Created: 2013-06-09 12:51\nLabels:                                                                                        Updated: 2013-06-27 21:02\n\nIssue Links:\n\nSub-Tasks:\n\nDescription:\nNone\n\nComments:'
__git_dirty_diff__ = ''